{-# LANGUAGE DataKinds #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TypeOperators #-}
{-# LANGUAGE MultiParamTypeClasses #-}

module BBW.API where

import Control.Monad.IO.Class
import Data.Proxy
import Network.HTTP.Client (newManager, Manager, defaultManagerSettings)
import BBW.Multipart
import Servant.API
import Servant.API.ContentTypes
import Servant.Client
import Servant.Multipart
import System.Environment

import qualified Network.HTTP.Media as M
import qualified Data.ByteString.Lazy as LBS
import qualified Data.Text as T
import qualified Data.Text.Lazy as TL
import qualified Data.Text.Lazy.Encoding as TL

-- was MultipartData Mem

newtype TEI = TEI T.Text

instance Accept TEI where
  contentType _ = "application" M.// "xml"

instance MimeUnrender TEI TEI where
  mimeUnrender _ b = Right . TEI . TL.toStrict $ TL.decodeUtf8 b

type GrobidAPI =
  "processHeaderDocument" :> MultipartForm Tmp (MultipartData Mem) :> Post '[TEI] TEI
  :<|> "processFulltextDocument" :> MultipartForm Tmp (MultipartData Mem) :> Post '[TEI] TEI

-- processHeaderDocument :: FilePath -> ClientM T.Text
processHeaderDocument :: MultipartData Mem -> ClientM TEI
processFulltextDocument :: MultipartData Mem -> ClientM TEI

grobidAPI :: Proxy GrobidAPI
grobidAPI = Proxy

processHeaderDocument :<|> processFulltextDocument = client grobidAPI

queries :: FilePath -> IO (Either ServantError TEI)
queries filePath = do
  fileContents <- liftIO $ LBS.readFile filePath
  manager <- newManager defaultManagerSettings
  let
    fileData = FileData
      { fdInputName = "input"
      , fdFileName  = T.pack filePath
      , fdFileCType = "application/pdf"
      , fdPayload   = fileContents
      }
    multipartData = MultipartData
      { inputs = []
      , files = [fileData]
      }
    c = processFulltextDocument multipartData
    baseUrl = BaseUrl Http "localhost" 8080 "api"
    env = mkClientEnv manager baseUrl
  runClientM c env
